﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace InteraktivniKlient
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        WebClient wc = new WebClient();
        int gameNumber = -1;
        int remainingMoves = 0;
        int totalMoves = 0;
        Player player;
        Player opponent;


        byte[,] field = new byte[10, 10];


        public MainWindow()
        {
            InitializeComponent();
        }

        private void Button_Click(object sender, RoutedEventArgs e)
        {
            Parse(wc.DownloadString(String.Format(@"http://{0}/start", txtServer.Text)));
            totalMoves = remainingMoves;
            Update();
        }

        void Update()
        {
            if (gameNumber == -1) return;
            txtRemainingMoves.Text = string.Format("{0}/{1}", remainingMoves, totalMoves);
            txtHeldTreasure.Text = player.P.ToString();
            txtSecuredTreasure.Text = player.C.ToString();
            btnDown.IsEnabled = (player.R != 9) && (field[player.R+1, player.S] < 253);
            btnUp.IsEnabled = (player.R != 0) && (field[player.R-1, player.S] < 253);
            btnLeft.IsEnabled = (player.S != 0) && (field[player.R, player.S-1] < 253);
            btnRight.IsEnabled = (player.S != 9) && (field[player.R, player.S+1] < 253);

            MainCanvas.Children.Clear();
            {
                for(int i = 0;i<10;i++)
                {
                    for(int j = 0;j<10;j++)
                    {
                        if(field[i,j] == 0) continue;
                        Image img = new Image() { Width = 10, Height = 10};
                        
                        Canvas.SetTop(img, i * 10);
                        Canvas.SetLeft(img, j * 10);

                        TextBlock block = null;

                        switch (field[i, j])
                        {
                            case 255:
                                img.Source = new BitmapImage(new Uri("pack://application:,,,/Resources/wall.png"));
                                break;
                            case 254:
                                img.Source = new BitmapImage(new Uri("pack://application:,,,/Resources/opponentbase.png"));
                                break;
                            case 253:
                                img.Source = new BitmapImage(new Uri("pack://application:,,,/Resources/opponent.png"));
                                break;
                            case 252:
                                img.Source = new BitmapImage(new Uri("pack://application:,,,/Resources/playerbase.png"));
                                break;
                            case 251:
                                img.Source = new BitmapImage(new Uri("pack://application:,,,/Resources/player.png"));
                                break;
                            default:
                                img.Source = new BitmapImage(new Uri("pack://application:,,,/Resources/treasure.png"));
                                block = new TextBlock() {Text = field[i, j].ToString(), Width = 10, Height = 7, TextAlignment = TextAlignment.Center, FontSize = 5};
                                Canvas.SetTop(block, i*10 + 3);
                                Canvas.SetLeft(block, j*10);
                                break;
                        }
                        MainCanvas.Children.Add(img);
                        if (block != null) MainCanvas.Children.Add(block);
                        
                    }
                }
            }
        }

        void Parse(string Response)
        {
            if (Response.Split('\n').Length < 13)
            {
                string[] answer = Response.Split(' ');

                if (answer[0] == "Chyba:")
                {
                    ShowError(Response);
                }
                else
                {
                    int playerResult = int.Parse(answer[2]);
                    if (answer.Length == 4)
                    {
                        int opponentResult = int.Parse(answer[3]);
                        if (opponentResult > playerResult)
                        {
                            EndGame(String.Format("Prohráli jste: {0} vs {1}.", playerResult, opponentResult), MessageBoxImage.Information);
                        }
                        else
                        {
                            EndGame(String.Format("Vyhráli jste: {0} vs {1}.", playerResult, opponentResult), MessageBoxImage.Information);
                        }
                    }
                    else
                    {
                        EndGame(String.Format("Dosažené skóre: {0}", playerResult), MessageBoxImage.Information);
                    }
                }
            }
            else
            {
                string[] lines = Response.Split('\n');

                gameNumber = int.Parse(lines[0].Trim());
                remainingMoves = int.Parse(lines[1].Trim());
                for (int i = 0; i < 10; i++)
                {
                    for (int j = 0; j < 10; j++)
                    {
                        char ch = lines[i + 2][j * 2];
                        if (ch == '.') field[i, j] = 0;
                        else if (ch == 'X') field[i, j] = 255;
                        else if (ch == '+') field[i, j] = 252;
                        else if (ch == '-') field[i, j] = 254;
                        else field[i, j] = byte.Parse(lines[i + 2].Substring(j * 2, 2));
                    }
                }

              

                player = new Player(lines[12].Trim());
                field[player.R, player.S] = 251;
                
                if (lines.Length > 14 && lines[13].Trim().Length > 5)
                {
                    opponent = new Player(lines[13].Trim());
                    field[opponent.R, opponent.S] = 253;
                }
                else
                {
                    opponent = null;
                }
            }
        }

        void ShowError(string Text)
        {
            EndGame(Text, MessageBoxImage.Error);
        }



        void EndGame(string Text, MessageBoxImage img)
        {
            MessageBox.Show(Text, "Treasure Hunters", MessageBoxButton.OK, img);
            
            foreach(UIElement el in controlsCanvas.Children)
            {
                el.IsEnabled = false;
            }
            gameNumber = -1;
        }

        private void Arrow_Click(object sender, RoutedEventArgs e)
        {
            Parse(wc.DownloadString("http://" + txtServer.Text + string.Format("/tah?id={0}&pohyb={1}", gameNumber, ((Button)sender).Tag)));
            Update();
        }

        private void window_KeyDown(object sender, KeyEventArgs e)
        {
            if (gameNumber == -1) return;
            char direct = '0';

            switch(e.Key)
            {
                case Key.Up:
                    if(btnUp.IsEnabled) direct = 'N';
                    break;
                case Key.Down:
                    if (btnDown.IsEnabled) direct = 'D';
                    break;
                case Key.Left:
                    if (btnLeft.IsEnabled) direct = 'L';
                    break;
                case Key.Right:
                    if (btnRight.IsEnabled) direct = 'P';
                    break;               
            }

            if(direct != '0')
            {
                Parse(wc.DownloadString("http://" + txtServer.Text + string.Format("/tah?id={0}&pohyb={1}", gameNumber, direct)));
                Update();
            }
        }
    }

    class Player
    {
        public int S;
        public int R;
        public int P;
        public int C;

        public Player(string PlayerInfo)
        {
            string[] Splitted = PlayerInfo.Split(' ');
            S = int.Parse(Splitted[0].Trim()) - 1;
            R = int.Parse(Splitted[1].Trim()) - 1;
            P = int.Parse(Splitted[2].Trim());
            C = int.Parse(Splitted[3].Trim());
        }
    }
}
